@@ -3,29 +3,28 @@ package commands
 import (
 	"context"
 	"fmt"
-	"os"
-	"strings"
-
 	"github.com/kluctl/kluctl/v2/cmd/kluctl/args"
 	"github.com/kluctl/kluctl/v2/pkg/deployment"
 	"github.com/kluctl/kluctl/v2/pkg/git"
 	"github.com/kluctl/kluctl/v2/pkg/git/auth"
-	git_url "github.com/kluctl/kluctl/v2/pkg/git/git-url"
 	"github.com/kluctl/kluctl/v2/pkg/git/messages"
 	ssh_pool "github.com/kluctl/kluctl/v2/pkg/git/ssh-pool"
 	"github.com/kluctl/kluctl/v2/pkg/kluctl_jinja2"
 	"github.com/kluctl/kluctl/v2/pkg/kluctl_project"
-	"github.com/kluctl/kluctl/v2/pkg/registries"
 	"github.com/kluctl/kluctl/v2/pkg/repocache"
+	"github.com/kluctl/kluctl/v2/pkg/results"
 	"github.com/kluctl/kluctl/v2/pkg/status"
+	"github.com/kluctl/kluctl/v2/pkg/types"
 	"github.com/kluctl/kluctl/v2/pkg/utils"
 	"github.com/kluctl/kluctl/v2/pkg/utils/uo"
 	"k8s.io/client-go/rest"
 	"k8s.io/client-go/tools/clientcmd"
 	"k8s.io/client-go/tools/clientcmd/api"
+	"os"
+	"strings"
 )
 
-func withKluctlProjectFromArgs(ctx context.Context, projectFlags args.ProjectFlags, argsFlags *args.ArgsFlags, strictTemplates bool, forCompletion bool, cb func(ctx context.Context, p *kluctl_project.LoadedKluctlProject) error) error {
+func withKluctlProjectFromArgs(ctx context.Context, projectFlags args.ProjectFlags, argsFlags *args.ArgsFlags, internalDeploy bool, strictTemplates bool, forCompletion bool, cb func(ctx context.Context, p *kluctl_project.LoadedKluctlProject) error) error {
 	tmpDir, err := os.MkdirTemp(utils.GetTmpBaseDir(ctx), "project-")
 	if err != nil {
 		return fmt.Errorf("creating temporary project directory failed: %w", err)
@@ -43,9 +42,16 @@ func withKluctlProjectFromArgs(ctx context.Context, projectFlags args.ProjectFla
 		return err
 	}
 
-	repoRoot, err := git.DetectGitRepositoryRoot(projectDir)
-	if err != nil {
-		status.Warning(ctx, "Failed to detect git project root. This might cause follow-up errors")
+	var repoRoot string
+	if !internalDeploy {
+		repoRoot, err = git.DetectGitRepositoryRoot(projectDir)
+		if err != nil {
+			status.Warning(ctx, "Failed to detect git project root. This might cause follow-up errors")
+		}
+	}
+
+	if repoRoot == "" {
+		repoRoot = projectDir
 	}
 
 	ctx, cancel := context.WithTimeout(ctx, projectFlags.Timeout)
@@ -125,7 +131,9 @@ type projectTargetCommandArgs struct {
 	helmCredentials      args.HelmCredentials
 	dryRunArgs           *args.DryRunFlags
 	renderOutputDirFlags args.RenderOutputDirFlags
+	commandResultFlags   *args.CommandResultFlags
 
+	internalDeploy    bool
 	forSeal           bool
 	forCompletion     bool
 	offlineKubernetes bool
@@ -133,30 +141,20 @@ type projectTargetCommandArgs struct {
 }
 
 type commandCtx struct {
-	ctx       context.Context
-	targetCtx *kluctl_project.TargetContext
-	images    *deployment.Images
+	ctx         context.Context
+	targetCtx   *kluctl_project.TargetContext
+	images      *deployment.Images
+	resultStore results.ResultStore
 }
 
 func withProjectCommandContext(ctx context.Context, args projectTargetCommandArgs, cb func(cmdCtx *commandCtx) error) error {
-	return withKluctlProjectFromArgs(ctx, args.projectFlags, &args.argsFlags, true, false, func(ctx context.Context, p *kluctl_project.LoadedKluctlProject) error {
+	return withKluctlProjectFromArgs(ctx, args.projectFlags, &args.argsFlags, args.internalDeploy, true, false, func(ctx context.Context, p *kluctl_project.LoadedKluctlProject) error {
 		return withProjectTargetCommandContext(ctx, args, p, cb)
 	})
 }
 
 func withProjectTargetCommandContext(ctx context.Context, args projectTargetCommandArgs, p *kluctl_project.LoadedKluctlProject, cb func(cmdCtx *commandCtx) error) error {
-	rh := registries.NewRegistryHelper(ctx)
-	err := rh.ParseAuthEntriesFromEnv()
-	if err != nil {
-		return fmt.Errorf("failed to parse registry auth from environment: %w", err)
-	}
-	if args.imageFlags.UpdateImages {
-		status.Deprecation(ctx, "update-images", "--update-images is deprecated and will be removed in the next kluctl release.")
-	}
-	if !args.imageFlags.OfflineImages {
-		status.Deprecation(ctx, "online-images", "--offline-images=false is deprecated and will be removed in the next kluctl release.")
-	}
-	images, err := deployment.NewImages(rh, args.imageFlags.UpdateImages, args.imageFlags.OfflineImages || args.forCompletion)
+	images, err := deployment.NewImages()
 	if err != nil {
 		return err
 	}
@@ -207,10 +205,24 @@ func withProjectTargetCommandContext(ctx context.Context, args projectTargetComm
 		}
 	}
 
+	var resultStore results.ResultStore
+	if args.commandResultFlags != nil && args.commandResultFlags.WriteCommandResult {
+		client, err := targetCtx.SharedContext.K.ToClient()
+		if err != nil {
+			return err
+		}
+
+		resultStore, err = results.NewResultStoreSecrets(ctx, client, nil, args.commandResultFlags.CommandResultNamespace, args.commandResultFlags.KeepCommandResultsCount)
+		if err != nil {
+			return err
+		}
+	}
+
 	cmdCtx := &commandCtx{
-		ctx:       ctx,
-		targetCtx: targetCtx,
-		images:    images,
+		ctx:         ctx,
+		targetCtx:   targetCtx,
+		images:      images,
+		resultStore: resultStore,
 	}
 
 	return cb(cmdCtx)
@@ -251,18 +263,17 @@ func parseRepoOverride(s string, isGroup bool) (ret repocache.RepoOverride, err
 		return repocache.RepoOverride{}, fmt.Errorf("%s", s)
 	}
 
-	u, err := git_url.Parse(sp[0])
+	u, err := types.ParseGitUrl(sp[0])
 	if err != nil {
 		// we need to prepend a dummy scheme to the repo key so that it is properly parsed
 		dummyUrl := fmt.Sprintf("git://%s", sp[0])
-		u, err = git_url.Parse(dummyUrl)
+		u, err = types.ParseGitUrl(dummyUrl)
 		if err != nil {
 			return repocache.RepoOverride{}, fmt.Errorf("%s: %w", s, err)
 		}
 	}
 
-	u = u.Normalize()
-	ret.RepoUrl = *u
+	ret.RepoKey = u.RepoKey()
 	ret.Override = sp[1]
 	return
 }
