@@ -1,16 +1,12 @@
 package test_utils
 
 import (
-	"bytes"
 	"context"
 	"fmt"
-	"github.com/fluxcd/go-git/v5"
+	"github.com/go-git/go-git/v5"
 	"github.com/huandu/xstrings"
 	"github.com/jinzhu/copier"
-	"github.com/kluctl/kluctl/v2/cmd/kluctl/commands"
 	git2 "github.com/kluctl/kluctl/v2/pkg/git"
-	"github.com/kluctl/kluctl/v2/pkg/status"
-	"github.com/kluctl/kluctl/v2/pkg/utils"
 	"github.com/kluctl/kluctl/v2/pkg/utils/uo"
 	"github.com/kluctl/kluctl/v2/pkg/yaml"
 	registry2 "helm.sh/helm/v3/pkg/registry"
@@ -22,7 +18,6 @@ import (
 	"path/filepath"
 	"reflect"
 	"strings"
-	"sync"
 	"testing"
 )
 
@@ -31,6 +26,7 @@ type TestProject struct {
 
 	extraEnv   []string
 	useProcess bool
+	bare       bool
 
 	gitServer   *git2.TestGitServer
 	gitRepoName string
@@ -63,6 +59,12 @@ func WithGitSubDir(subDir string) TestProjectOption {
 	}
 }
 
+func WithBareProject() TestProjectOption {
+	return func(p *TestProject) {
+		p.bare = true
+	}
+}
+
 func NewTestProject(t *testing.T, opts ...TestProjectOption) *TestProject {
 	p := &TestProject{
 		t:           t,
@@ -78,13 +80,15 @@ func NewTestProject(t *testing.T, opts ...TestProjectOption) *TestProject {
 	}
 	p.gitServer.GitInit(p.gitRepoName)
 
-	p.UpdateKluctlYaml(func(o *uo.UnstructuredObject) error {
-		_ = o.SetNestedField(fmt.Sprintf("%s-{{ target.name or 'no-name' }}", rand.String(16)), "discriminator")
-		return nil
-	})
-	p.UpdateDeploymentYaml(".", func(c *uo.UnstructuredObject) error {
-		return nil
-	})
+	if !p.bare {
+		p.UpdateKluctlYaml(func(o *uo.UnstructuredObject) error {
+			_ = o.SetNestedField(fmt.Sprintf("%s-{{ target.name or 'no-name' }}", rand.String(16)), "discriminator")
+			return nil
+		})
+		p.UpdateDeploymentYaml(".", func(c *uo.UnstructuredObject) error {
+			return nil
+		})
+	}
 	return p
 }
 
@@ -459,38 +463,7 @@ func (p *TestProject) KluctlExecute(argsIn ...string) (string, string, error) {
 	args = append(args, "--project-dir", p.LocalProjectDir())
 	args = append(args, argsIn...)
 
-	p.t.Logf("Runnning kluctl: %s", strings.Join(args, " "))
-
-	var m sync.Mutex
-	stdoutBuf := bytes.NewBuffer(nil)
-	stdout := status.NewLineRedirector(func(line string) {
-		m.Lock()
-		defer m.Unlock()
-		p.t.Log(line)
-		stdoutBuf.WriteString(line + "\n")
-	})
-	stderrBuf := bytes.NewBuffer(nil)
-
-	ctx := context.Background()
-	ctx = utils.WithTmpBaseDir(ctx, p.t.TempDir())
-	ctx = commands.WithStdStreams(ctx, stdout, stderrBuf)
-	sh := status.NewSimpleStatusHandler(func(message string) {
-		m.Lock()
-		defer m.Unlock()
-		p.t.Log(message)
-		stderrBuf.WriteString(message + "\n")
-	}, false, true)
-	defer func() {
-		if sh != nil {
-			sh.Stop()
-		}
-	}()
-	ctx = status.NewContext(ctx, sh)
-	err := commands.Execute(ctx, args, nil)
-	sh.Stop()
-	sh = nil
-	_ = stdout.Close()
-	return stdoutBuf.String(), stderrBuf.String(), err
+	return KluctlExecute(p.t, context.Background(), args...)
 }
 
 func (p *TestProject) Kluctl(argsIn ...string) (string, string, error) {
