@@ -2,11 +2,13 @@ package utils
 
 import (
 	"context"
+	"encoding/json"
 	"github.com/kluctl/kluctl/v2/pkg/deployment"
-	"github.com/kluctl/kluctl/v2/pkg/types"
 	k8s2 "github.com/kluctl/kluctl/v2/pkg/types/k8s"
+	"github.com/kluctl/kluctl/v2/pkg/types/result"
 	"github.com/kluctl/kluctl/v2/pkg/utils/uo"
 	"github.com/stretchr/testify/assert"
+	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	"testing"
 )
 
@@ -19,7 +21,7 @@ type diffTestConfig struct {
 
 	dew *DeploymentErrorsAndWarnings
 	ru  *RemoteObjectUtils
-	du  *diffUtil
+	du  *DiffUtil
 }
 
 func (dtc *diffTestConfig) newRemoteObjects(dew *DeploymentErrorsAndWarnings) *RemoteObjectUtils {
@@ -56,6 +58,28 @@ func newTestConfigMap(name string, data map[string]interface{}) *uo.Unstructured
 }
 
 func TestDiff(t *testing.T) {
+	buildRaw := func(x any) *apiextensionsv1.JSON {
+		if x == nil {
+			return nil
+		}
+		b, err := json.Marshal(x)
+		if err != nil {
+			t.Fatal(err)
+		}
+		return &apiextensionsv1.JSON{
+			Raw: b,
+		}
+	}
+	buildChange := func(typ string, jsonPath string, oldValue any, newValue any, unifiedDiff string) result.Change {
+		return result.Change{
+			Type:        typ,
+			JsonPath:    jsonPath,
+			NewValue:    buildRaw(newValue),
+			OldValue:    buildRaw(oldValue),
+			UnifiedDiff: unifiedDiff,
+		}
+	}
+
 	tests := []*diffTestConfig{
 		{
 			name: "One changed object (changed field)",
@@ -64,9 +88,8 @@ func TestDiff(t *testing.T) {
 			ao:   []*uo.UnstructuredObject{newTestConfigMap("test", map[string]interface{}{"d1": "v2"})},
 			a: func(t *testing.T, dtc *diffTestConfig) {
 				assert.Len(t, dtc.du.ChangedObjects, 1)
-				assert.Equal(t, dtc.du.ChangedObjects[0].NewObject, dtc.ao[0])
-				assert.Equal(t, []types.Change{
-					types.Change{Type: "update", JsonPath: "data.d1", OldValue: "v1", NewValue: "v2", UnifiedDiff: "-v1\n+v2"},
+				assert.Equal(t, []result.Change{
+					buildChange("update", "data.d1", buildRaw("v1"), buildRaw("v2"), "-v1\n+v2"),
 				}, dtc.du.ChangedObjects[0].Changes)
 			},
 		},
@@ -77,9 +100,8 @@ func TestDiff(t *testing.T) {
 			ao:   []*uo.UnstructuredObject{newTestConfigMap("test", map[string]interface{}{"d1": "v1", "d2": "v2"})},
 			a: func(t *testing.T, dtc *diffTestConfig) {
 				assert.Len(t, dtc.du.ChangedObjects, 1)
-				assert.Equal(t, dtc.du.ChangedObjects[0].NewObject, dtc.ao[0])
-				assert.Equal(t, []types.Change{
-					types.Change{Type: "insert", JsonPath: "data.d2", OldValue: interface{}(nil), NewValue: "v2", UnifiedDiff: "+v2"},
+				assert.Equal(t, []result.Change{
+					buildChange("insert", "data.d2", nil, buildRaw("v2"), "+v2"),
 				}, dtc.du.ChangedObjects[0].Changes)
 			},
 		},
@@ -90,9 +112,8 @@ func TestDiff(t *testing.T) {
 			ao:   []*uo.UnstructuredObject{newTestConfigMap("test", map[string]interface{}{"d1": "v1"})},
 			a: func(t *testing.T, dtc *diffTestConfig) {
 				assert.Len(t, dtc.du.ChangedObjects, 1)
-				assert.Equal(t, dtc.du.ChangedObjects[0].NewObject, dtc.ao[0])
-				assert.Equal(t, []types.Change{
-					types.Change{Type: "delete", JsonPath: "data.d2", OldValue: "v2", NewValue: interface{}(nil), UnifiedDiff: "-v2"},
+				assert.Equal(t, []result.Change{
+					buildChange("delete", "data.d2", buildRaw("v2"), nil, "-v2"),
 				}, dtc.du.ChangedObjects[0].Changes)
 			},
 		},
@@ -103,10 +124,9 @@ func TestDiff(t *testing.T) {
 			ao:   []*uo.UnstructuredObject{newTestConfigMap("test", map[string]interface{}{"d2": "v2"})},
 			a: func(t *testing.T, dtc *diffTestConfig) {
 				assert.Len(t, dtc.du.ChangedObjects, 1)
-				assert.Equal(t, dtc.du.ChangedObjects[0].NewObject, dtc.ao[0])
-				assert.Equal(t, []types.Change{
-					types.Change{Type: "delete", JsonPath: "data.d1", OldValue: "v1", NewValue: interface{}(nil), UnifiedDiff: "-v1"},
-					types.Change{Type: "insert", JsonPath: "data.d2", OldValue: interface{}(nil), NewValue: "v2", UnifiedDiff: "+v2"},
+				assert.Equal(t, []result.Change{
+					buildChange("delete", "data.d1", buildRaw("v1"), nil, "-v1"),
+					buildChange("insert", "data.d2", nil, buildRaw("v2"), "+v2"),
 				}, dtc.du.ChangedObjects[0].Changes)
 			},
 		},
@@ -117,11 +137,10 @@ func TestDiff(t *testing.T) {
 			ao:   []*uo.UnstructuredObject{newTestConfigMap("test", map[string]interface{}{"d1": "v12", "d3": "v3"})},
 			a: func(t *testing.T, dtc *diffTestConfig) {
 				assert.Len(t, dtc.du.ChangedObjects, 1)
-				assert.Equal(t, dtc.du.ChangedObjects[0].NewObject, dtc.ao[0])
-				assert.Equal(t, []types.Change{
-					types.Change{Type: "delete", JsonPath: "data.d2", OldValue: "v2", NewValue: interface{}(nil), UnifiedDiff: "-v2"},
-					types.Change{Type: "insert", JsonPath: "data.d3", OldValue: interface{}(nil), NewValue: "v3", UnifiedDiff: "+v3"},
-					types.Change{Type: "update", JsonPath: "data.d1", OldValue: "v1", NewValue: "v12", UnifiedDiff: "-v1\n+v12"},
+				assert.Equal(t, []result.Change{
+					buildChange("update", "data.d1", buildRaw("v1"), buildRaw("v12"), "-v1\n+v12"),
+					buildChange("delete", "data.d2", buildRaw("v2"), nil, "-v2"),
+					buildChange("insert", "data.d3", nil, buildRaw("v3"), "+v3"),
 				}, dtc.du.ChangedObjects[0].Changes)
 			},
 		},
@@ -131,8 +150,8 @@ func TestDiff(t *testing.T) {
 		t.Run(test.name, func(t *testing.T) {
 			test.dew = NewDeploymentErrorsAndWarnings()
 			test.ru = test.newRemoteObjects(test.dew)
-			test.du = NewDiffUtil(test.dew, test.newDeploymentItems(), test.ru, test.appliedObjectsMap())
-			test.du.Diff()
+			test.du = NewDiffUtil(test.dew, test.ru, test.appliedObjectsMap())
+			test.du.DiffDeploymentItems(test.newDeploymentItems())
 			test.a(t, test)
 		})
 	}
