@@ -2,14 +2,14 @@ package git
 
 import (
 	"fmt"
-	"github.com/fluxcd/go-git/v5"
+	"github.com/go-git/go-git/v5"
 	"os"
 	"path/filepath"
 )
 
 type CheckoutInfo struct {
-	CheckedOutRef    string `yaml:"checkedOutRef"`
-	CheckedOutCommit string `yaml:"checkedOutCommit"`
+	CheckedOutRef    string `json:"checkedOutRef"`
+	CheckedOutCommit string `json:"checkedOutCommit"`
 }
 
 func GetCheckoutInfo(path string) (ri CheckoutInfo, err error) {
