@@ -2,10 +2,12 @@ package k8s
 
 import (
 	"context"
-	"encoding/json"
 	"fmt"
 	"io"
+	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
+	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"net/http"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 	"strings"
 	"sync"
 	"time"
@@ -16,18 +18,14 @@ import (
 	"github.com/kluctl/kluctl/v2/pkg/types/k8s"
 	"github.com/kluctl/kluctl/v2/pkg/utils"
 	"github.com/kluctl/kluctl/v2/pkg/utils/uo"
-	"github.com/kluctl/kluctl/v2/pkg/yaml"
 	"k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/meta"
 	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime/schema"
-	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/version"
 	"k8s.io/client-go/discovery"
-	"k8s.io/client-go/dynamic"
 	_ "k8s.io/client-go/plugin/pkg/client/auth"
 	"k8s.io/client-go/rest"
-	"k8s.io/client-go/restmapper"
 )
 
 type K8sCluster struct {
@@ -36,23 +34,26 @@ type K8sCluster struct {
 	DryRun bool
 
 	clientFactory ClientFactory
-	clients       *k8sClients
+
+	discovery      discovery.DiscoveryInterface
+	discoveryMutex *sync.Mutex
+
+	clients *k8sClients
 
 	ServerVersion *version.Info
-
-	Resources *k8sResources
 }
 
 func NewK8sCluster(ctx context.Context, clientFactory ClientFactory, dryRun bool) (*K8sCluster, error) {
 	var err error
 
 	k := &K8sCluster{
-		ctx:           ctx,
-		DryRun:        dryRun,
-		clientFactory: clientFactory,
+		ctx:            ctx,
+		DryRun:         dryRun,
+		clientFactory:  clientFactory,
+		discoveryMutex: &sync.Mutex{},
 	}
 
-	k.Resources, err = newK8sResources(ctx, clientFactory)
+	k.discovery, err = clientFactory.DiscoveryClient()
 	if err != nil {
 		return nil, err
 	}
@@ -62,34 +63,12 @@ func NewK8sCluster(ctx context.Context, clientFactory ClientFactory, dryRun bool
 		return nil, err
 	}
 
-	v, err := k.Resources.discovery.ServerVersion()
+	v, err := k.discovery.ServerVersion()
 	if err != nil {
 		return nil, err
 	}
 	k.ServerVersion = v
 
-	var wg sync.WaitGroup
-	wg.Add(2)
-
-	var err1 error
-	var err2 error
-	go func() {
-		err1 = k.Resources.updateResources()
-		wg.Done()
-	}()
-	go func() {
-		err2 = k.Resources.updateResourcesFromCRDs(k.clients)
-		wg.Done()
-	}()
-	wg.Wait()
-
-	if err1 != nil {
-		return nil, err1
-	}
-	if err2 != nil {
-		return nil, err2
-	}
-
 	return k, nil
 }
 
@@ -119,37 +98,61 @@ func (k *K8sCluster) buildLabelSelector(labels map[string]string) string {
 	return ret
 }
 
-func (k *K8sCluster) ListObjects(gvk schema.GroupVersionKind, namespace string, labels map[string]string) ([]*uo.UnstructuredObject, []ApiWarning, error) {
-	var result []*uo.UnstructuredObject
-
-	apiWarnings, err := k.clients.withDynamicClientForGVK(k.Resources, gvk, namespace, func(r dynamic.ResourceInterface) error {
-		o := v1.ListOptions{
-			LabelSelector: k.buildLabelSelector(labels),
-		}
-		x, err := r.List(k.ctx, o)
-		if err != nil {
-			return err
-		}
-		for _, o := range x.Items {
-			result = append(result, uo.FromUnstructured(&o))
-		}
-		return nil
+func (k *K8sCluster) doList(l client.ObjectList, namespace string, labels map[string]string) ([]*uo.UnstructuredObject, []ApiWarning, error) {
+	apiWarnings, err := k.clients.withCClientFromPool(true, func(c client.Client) error {
+		return c.List(k.ctx, l, client.InNamespace(namespace), client.MatchingLabels(labels))
 	})
+	if err != nil {
+		return nil, apiWarnings, err
+	}
+
+	items, err := meta.ExtractList(l)
+	if err != nil {
+		return nil, apiWarnings, err
+	}
+
+	result := make([]*uo.UnstructuredObject, len(items))
+	for i, o := range items {
+		if u, ok := o.(*unstructured.Unstructured); ok {
+			result[i] = uo.FromUnstructured(u)
+		} else {
+			x, err := uo.FromStruct(o)
+			if err != nil {
+				return nil, apiWarnings, err
+			}
+			result[i] = x
+		}
+	}
+
 	return result, apiWarnings, err
 }
+func (k *K8sCluster) ListObjects(gvk schema.GroupVersionKind, namespace string, labels map[string]string) ([]*uo.UnstructuredObject, []ApiWarning, error) {
+	var l unstructured.UnstructuredList
+	l.SetGroupVersionKind(gvk)
+	return k.doList(&l, namespace, labels)
+}
+
+func (k *K8sCluster) ListMetadata(gvk schema.GroupVersionKind, namespace string, labels map[string]string) ([]*uo.UnstructuredObject, []ApiWarning, error) {
+	var l v1.PartialObjectMetadataList
+	l.SetGroupVersionKind(gvk)
+	return k.doList(&l, namespace, labels)
+}
 
 func (k *K8sCluster) GetSingleObject(ref k8s.ObjectRef) (*uo.UnstructuredObject, []ApiWarning, error) {
-	var result *uo.UnstructuredObject
-	apiWarnings, err := k.clients.withDynamicClientForGVK(k.Resources, ref.GVK, ref.Namespace, func(r dynamic.ResourceInterface) error {
-		o := v1.GetOptions{}
-		x, err := r.Get(k.ctx, ref.Name, o)
-		if err != nil {
-			return err
-		}
-		result = uo.FromUnstructured(x)
-		return nil
+	var o unstructured.Unstructured
+	o.SetGroupVersionKind(ref.GroupVersionKind())
+
+	apiWarnings, err := k.clients.withCClientFromPool(true, func(c client.Client) error {
+		return c.Get(k.ctx, client.ObjectKey{
+			Name:      ref.Name,
+			Namespace: ref.Namespace,
+		}, &o)
+
 	})
-	return result, apiWarnings, err
+	if err != nil {
+		return nil, apiWarnings, err
+	}
+	return uo.FromUnstructured(&o), apiWarnings, nil
 }
 
 type DeleteOptions struct {
@@ -161,25 +164,19 @@ type DeleteOptions struct {
 func (k *K8sCluster) DeleteSingleObject(ref k8s.ObjectRef, options DeleteOptions) ([]ApiWarning, error) {
 	dryRun := k.DryRun || options.ForceDryRun
 
-	pp := v1.DeletePropagationBackground
-	o := v1.DeleteOptions{
-		PropagationPolicy: &pp,
-	}
-	if dryRun {
-		o.DryRun = []string{"All"}
-	}
+	var o unstructured.Unstructured
+	o.SetGroupVersionKind(ref.GroupVersionKind())
+	o.SetName(ref.Name)
+	o.SetNamespace(ref.Namespace)
 
-	apiWarnings, err := k.clients.withDynamicClientForGVK(k.Resources, ref.GVK, ref.Namespace, func(r dynamic.ResourceInterface) error {
-		err := r.Delete(k.ctx, ref.Name, o)
-		if err != nil {
-			if options.IgnoreNotFoundError && errors.IsNotFound(err) {
-				return nil
-			}
-			return err
-		}
-		return nil
+	apiWarnings, err := k.clients.withCClientFromPool(dryRun, func(c client.Client) error {
+		return c.Delete(k.ctx, &o, client.PropagationPolicy(v1.DeletePropagationBackground))
 	})
+
 	if err != nil {
+		if options.IgnoreNotFoundError && errors.IsNotFound(err) {
+			return apiWarnings, nil
+		}
 		return apiWarnings, err
 	}
 
@@ -189,6 +186,7 @@ func (k *K8sCluster) DeleteSingleObject(ref k8s.ObjectRef, options DeleteOptions
 			return apiWarnings, err
 		}
 	}
+
 	return apiWarnings, nil
 }
 
@@ -306,54 +304,35 @@ type PatchOptions struct {
 	ForceApply  bool
 }
 
-func (k *K8sCluster) doPatch(ref k8s.ObjectRef, data []byte, patchType types.PatchType, options PatchOptions) (*uo.UnstructuredObject, []ApiWarning, error) {
-	dryRun := k.DryRun || options.ForceDryRun
-
-	po := v1.PatchOptions{
-		FieldManager: "kluctl",
-	}
-	if dryRun {
-		po.DryRun = []string{"All"}
-	}
-	if options.ForceApply {
-		po.Force = &options.ForceApply
-	}
-
+func (k *K8sCluster) doPatch(ref k8s.ObjectRef, obj client.Object, patch client.Patch, options PatchOptions) ([]ApiWarning, error) {
 	status.Trace(k.ctx, "patching %s", ref.String())
 
-	var result *uo.UnstructuredObject
-	apiWarnings, err := k.clients.withDynamicClientForGVK(k.Resources, ref.GVK, ref.Namespace, func(r dynamic.ResourceInterface) error {
-		x, err := r.Patch(k.ctx, ref.Name, patchType, data, po)
+	var opts []client.PatchOption
+	if options.ForceApply {
+		opts = append(opts, client.ForceOwnership)
+	}
+	if options.ForceDryRun {
+		opts = append(opts, client.DryRunAll)
+	}
+	opts = append(opts, client.FieldOwner("kluctl"))
+
+	apiWarnings, err := k.clients.withCClientFromPool(k.DryRun, func(c client.Client) error {
+		err := c.Patch(k.ctx, obj, patch, opts...)
 		if err != nil {
 			return fmt.Errorf("failed to patch %s: %w", ref.String(), err)
 		}
-		result = uo.FromUnstructured(x)
 		return nil
 	})
-	return result, apiWarnings, err
+	return apiWarnings, err
 }
 
-func (k *K8sCluster) PatchObject(o *uo.UnstructuredObject, options PatchOptions) (*uo.UnstructuredObject, []ApiWarning, error) {
-	data, err := yaml.WriteYamlBytes(o)
+func (k *K8sCluster) ApplyObject(o *uo.UnstructuredObject, options PatchOptions) (*uo.UnstructuredObject, []ApiWarning, error) {
+	obj := o.Clone().ToUnstructured()
+	apiWarnings, err := k.doPatch(o.GetK8sRef(), obj, client.Apply, options)
 	if err != nil {
-		return nil, nil, err
+		return nil, apiWarnings, err
 	}
-
-	return k.doPatch(o.GetK8sRef(), data, types.ApplyPatchType, options)
-}
-
-type JsonPatch struct {
-	Op    string `json:"op"`
-	Path  string `json:"path"`
-	Value any    `json:"value"`
-}
-
-func (k *K8sCluster) PatchObjectWithJsonPatch(ref k8s.ObjectRef, patch interface{}, options PatchOptions) (*uo.UnstructuredObject, []ApiWarning, error) {
-	data, err := json.Marshal(patch)
-	if err != nil {
-		return nil, nil, err
-	}
-	return k.doPatch(ref, data, types.JSONPatchType, options)
+	return uo.FromUnstructured(obj), apiWarnings, nil
 }
 
 type UpdateOptions struct {
@@ -361,28 +340,24 @@ type UpdateOptions struct {
 }
 
 func (k *K8sCluster) UpdateObject(o *uo.UnstructuredObject, options UpdateOptions) (*uo.UnstructuredObject, []ApiWarning, error) {
-	dryRun := k.DryRun || options.ForceDryRun
 	ref := o.GetK8sRef()
-
-	updateOpts := v1.UpdateOptions{
-		FieldManager: "kluctl",
-	}
-	if dryRun {
-		updateOpts.DryRun = []string{"All"}
-	}
+	obj := o.Clone().ToUnstructured()
 
 	status.Trace(k.ctx, "updating %s", ref.String())
 
-	var result *uo.UnstructuredObject
-	apiWarnings, err := k.clients.withDynamicClientForGVK(k.Resources, ref.GVK, ref.Namespace, func(r dynamic.ResourceInterface) error {
-		x, err := r.Update(k.ctx, o.ToUnstructured(), updateOpts)
-		if err != nil {
-			return err
-		}
-		result = uo.FromUnstructured(x)
-		return nil
+	var opts []client.UpdateOption
+	if options.ForceDryRun {
+		opts = append(opts, client.DryRunAll)
+	}
+	opts = append(opts, client.FieldOwner("kluctl"))
+
+	apiWarnings, err := k.clients.withCClientFromPool(k.DryRun, func(c client.Client) error {
+		return c.Update(k.ctx, obj, opts...)
 	})
-	return result, apiWarnings, err
+	if err != nil {
+		return nil, apiWarnings, err
+	}
+	return uo.FromUnstructured(obj), apiWarnings, nil
 }
 
 // envtestProxyGet checks the environment variables KLUCTL_K8S_SERVICE_PROXY_XXX to enable testing of proxy requests with envtest
@@ -435,16 +410,116 @@ func (k *K8sCluster) ProxyGet(scheme, namespace, name, port, path string, params
 	return ret.Stream(k.ctx)
 }
 
+func (k *K8sCluster) IsNamespaced(gvk schema.GroupVersionKind) *bool {
+	var obj unstructured.Unstructured
+	obj.SetGroupVersionKind(gvk)
+
+	ret := false
+	_, err := k.clients.withCClientFromPool(true, func(c client.Client) error {
+		x, err := c.IsObjectNamespaced(&obj)
+		if err != nil {
+			return err
+		}
+		ret = x
+		return nil
+	})
+	if err != nil {
+		return nil
+	}
+	return &ret
+}
+
+func (k *K8sCluster) FixNamespace(o *uo.UnstructuredObject, def string) {
+	ref := o.GetK8sRef()
+	namespaced := k.IsNamespaced(ref.GroupVersionKind())
+	if namespaced == nil {
+		return
+	}
+	if !*namespaced && ref.Namespace != "" {
+		o.SetK8sNamespace("")
+	} else if *namespaced && ref.Namespace == "" {
+		o.SetK8sNamespace(def)
+	}
+}
+
+func (k *K8sCluster) FixNamespaceInRef(ref k8s.ObjectRef) k8s.ObjectRef {
+	namespaced := k.IsNamespaced(ref.GroupVersionKind())
+	if namespaced == nil {
+		return ref
+	}
+	if !*namespaced && ref.Namespace != "" {
+		ref.Namespace = ""
+	} else if *namespaced && ref.Namespace == "" {
+		ref.Namespace = "default"
+	}
+	return ref
+}
+
+func (k *K8sCluster) GetSchemaForGVK(gvk schema.GroupVersionKind) (*uo.UnstructuredObject, error) {
+	rms, err := k.clientFactory.Mapper().RESTMappings(gvk.GroupKind(), gvk.Version)
+	if err != nil {
+		return nil, err
+	}
+
+	var rm *meta.RESTMapping
+	for _, x := range rms {
+		if x.GroupVersionKind == gvk {
+			rm = x
+			break
+		}
+	}
+	if rm == nil {
+		return nil, fmt.Errorf("rest mapping not found")
+	}
+
+	ref := k8s.NewObjectRef(apiextensionsv1.GroupName, "v1", "CustomResourceDefinition", fmt.Sprintf("%s.%s", rm.Resource.Resource, gvk.Group), "")
+
+	crd, _, err := k.GetSingleObject(ref)
+	if err != nil {
+		return nil, err
+	}
+
+	versions, ok, err := crd.GetNestedObjectList("spec", "versions")
+	if err != nil {
+		return nil, err
+	}
+	if !ok {
+		return nil, fmt.Errorf("versions not found in CRD")
+	}
+
+	for _, v := range versions {
+		name, _, _ := v.GetNestedString("name")
+		if name != gvk.Version {
+			continue
+		}
+		s, ok, err := v.GetNestedObject("schema", "openAPIV3Schema")
+		if err != nil {
+			return nil, err
+		}
+		if !ok {
+			return nil, fmt.Errorf("version %s has no schema", name)
+		}
+		return s, nil
+	}
+	return nil, fmt.Errorf("schema for %s not found", gvk.String())
+}
+
+func (k *K8sCluster) ResetMapper() {
+	if m, ok := k.clientFactory.Mapper().(meta.ResettableRESTMapper); ok {
+		m.Reset()
+	}
+}
+
+func (k *K8sCluster) ToClient() (client.Client, error) {
+	return k.clientFactory.Client(nil)
+}
+
 func (k *K8sCluster) ToRESTConfig() (*rest.Config, error) {
 	return k.clientFactory.RESTConfig(), nil
 }
 
 func (k *K8sCluster) ToDiscoveryClient() (discovery.CachedDiscoveryInterface, error) {
-	d, err := k.clientFactory.DiscoveryClient()
-	if err != nil {
-		return nil, err
-	}
-	cd, ok := d.(discovery.CachedDiscoveryInterface)
+	cd, ok := k.discovery.(discovery.CachedDiscoveryInterface)
 	if !ok {
 		return nil, fmt.Errorf("not a CachedDiscoveryInterface")
 	}
@@ -452,12 +527,5 @@ func (k *K8sCluster) ToDiscoveryClient() (discovery.CachedDiscoveryInterface, er
 }
 
 func (k *K8sCluster) ToRESTMapper() (meta.RESTMapper, error) {
-	discoveryClient, err := k.ToDiscoveryClient()
-	if err != nil {
-		return nil, err
-	}
-
-	mapper := restmapper.NewDeferredDiscoveryRESTMapper(discoveryClient)
-	expander := restmapper.NewShortcutExpander(mapper, discoveryClient)
-	return expander, nil
+	return k.clientFactory.Mapper(), nil
 }
