@@ -18,10 +18,12 @@ import (
 )
 
 type TargetContext struct {
+	Params TargetContextParams
+
 	SharedContext deployment.SharedContext
 
 	KluctlProject        *LoadedKluctlProject
-	Target               *types.Target
+	Target               types.Target
 	ClusterContext       string
 	DeploymentProject    *deployment.DeploymentProject
 	DeploymentCollection *deployment.DeploymentCollection
@@ -42,7 +44,11 @@ type TargetContextParams struct {
 }
 
 func (p *LoadedKluctlProject) NewTargetContext(ctx context.Context, params TargetContextParams) (*TargetContext, error) {
-	deploymentDir, err := filepath.Abs(p.ProjectDir)
+	repoRoot, err := filepath.Abs(p.LoadArgs.RepoRoot)
+	if err != nil {
+		return nil, err
+	}
+	relProjectDir, err := filepath.Rel(repoRoot, p.LoadArgs.ProjectDir)
 	if err != nil {
 		return nil, err
 	}
@@ -50,11 +56,11 @@ func (p *LoadedKluctlProject) NewTargetContext(ctx context.Context, params Targe
 	var target *types.Target
 	needRender := false
 	if params.TargetName != "" {
-		t, err := p.FindDynamicTarget(params.TargetName)
+		t, err := p.FindTarget(params.TargetName)
 		if err != nil {
 			return nil, err
 		}
-		target = &*t.Target
+		target = &*t
 	} else {
 		target = &types.Target{
 			Discriminator: p.Config.Discriminator,
@@ -127,7 +133,7 @@ func (p *LoadedKluctlProject) NewTargetContext(ctx context.Context, params Targe
 		DefaultSealedSecretsOutputPattern: target.Name,
 	}
 
-	d, err := deployment.NewDeploymentProject(dctx, varsCtx, deployment.NewSource(deploymentDir), ".", nil)
+	d, err := deployment.NewDeploymentProject(dctx, varsCtx, deployment.NewSource(repoRoot), relProjectDir, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -138,9 +144,10 @@ func (p *LoadedKluctlProject) NewTargetContext(ctx context.Context, params Targe
 	}
 
 	targetCtx := &TargetContext{
+		Params:               params,
 		SharedContext:        dctx,
 		KluctlProject:        p,
-		Target:               target,
+		Target:               *target,
 		ClusterContext:       clusterContext,
 		DeploymentProject:    d,
 		DeploymentCollection: c,
@@ -159,7 +166,7 @@ func (p *LoadedKluctlProject) loadK8sConfig(target *types.Target, offlineK8s boo
 	var err error
 	var clientConfig *rest.Config
 	var restConfig *api.Config
-	clientConfig, restConfig, err = p.loadArgs.ClientConfigGetter(contextName)
+	clientConfig, restConfig, err = p.LoadArgs.ClientConfigGetter(contextName)
 	if err != nil {
 		return nil, "", err
 	}
@@ -193,8 +200,8 @@ func (p *LoadedKluctlProject) buildVars(target *types.Target, forSeal bool) (*va
 			}
 		}
 	}
-	if p.loadArgs.ExternalArgs != nil {
-		allArgs.Merge(p.loadArgs.ExternalArgs)
+	if p.LoadArgs.ExternalArgs != nil {
+		allArgs.Merge(p.LoadArgs.ExternalArgs)
 	}
 
 	err = deployment.LoadDefaultArgs(p.Config.Args, allArgs)
@@ -202,15 +209,6 @@ func (p *LoadedKluctlProject) buildVars(target *types.Target, forSeal bool) (*va
 		return nil, err
 	}
 
-	deprecatedArgs, err := deployment.LoadDeprecatedDeploymentArgs(p.ctx, p.ProjectDir, varsCtx, allArgs)
-	if err != nil {
-		return nil, err
-	}
-
-	if deprecatedArgs && len(p.Config.Args) != 0 {
-		return nil, fmt.Errorf("mixing deprecated 'args' from deployment.yaml and .kluctl.yaml is not allowed")
-	}
-
 	varsCtx.UpdateChild("args", allArgs)
 
 	return varsCtx, nil
@@ -226,7 +224,7 @@ func (p *LoadedKluctlProject) findSecretsEntry(name string) (*types.SecretSet, e
 }
 
 func (p *LoadedKluctlProject) loadSecrets(target *types.Target, varsCtx *vars.VarsCtx, varsLoader *vars.VarsLoader) error {
-	searchDirs := []string{p.ProjectDir}
+	searchDirs := []string{p.LoadArgs.ProjectDir}
 
 	for _, secretSetName := range target.SealingConfig.SecretSets {
 		secretEntry, err := p.findSecretsEntry(secretSetName)
